@@ -71,6 +71,14 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "print",
+    signature = signature(x = 'ideal'),
+    definition = function(x, ...){
+        cat('Item object of class:', class(x))
+    }
+)
+
 setMethod(
     f = "print",
     signature = signature(x = 'GroupPars'),
@@ -151,6 +159,14 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "show",
+    signature = signature(object = 'ideal'),
+    definition = function(object){
+        print(object)
+    }
+)
+
 setMethod(
     f = "show",
     signature = signature(object = 'GroupPars'),
@@ -234,6 +250,14 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "ExtractLambdas",
+    signature = signature(x = 'ideal'),
+    definition = function(x){
+        x@par[1L:x@nfact]
+    }
+)
+
 #----------------------------------------------------------------------------
 setMethod(
     f = "ExtractZetas",
@@ -303,6 +327,15 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "ExtractZetas",
+    signature = signature(x = 'ideal'),
+    definition = function(x){
+        d <- x@par[length(x@par)]
+        d
+    }
+)
+
 #----------------------------------------------------------------------------
 setMethod(
     f = "GenRandomPars",
@@ -367,7 +400,7 @@ setMethod(
     f = "GenRandomPars",
     signature = signature(x = 'gpcm'),
     definition = function(x){
-        par <- c(rlnorm(x@nfact, meanlog=-.2, sdlog=.5), 0,
+        par <- c(rlnorm(x@nfact, meanlog=-.2, sdlog=.5), 0:(x@ncat-1), 0,
                  sort(rnorm(x@ncat-1L, sd = 2), decreasing=TRUE))
         x@par[x@est] <- par[x@est]
         x
@@ -378,7 +411,7 @@ setMethod(
     f = "GenRandomPars",
     signature = signature(x = 'rsm'),
     definition = function(x){
-        par <- c(rlnorm(x@nfact, meanlog=0, sdlog=.5), 0,
+        par <- c(rlnorm(x@nfact, meanlog=0, sdlog=.5), 0:(x@ncat-1), 0,
                  sort(rnorm(x@ncat-1L, sd = 2), decreasing=TRUE),
                  rnorm(1L))
         x@par[x@est] <- par[x@est]
@@ -411,6 +444,17 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "GenRandomPars",
+    signature = signature(x = 'ideal'),
+    definition = function(x){
+        par <- c(rlnorm(x@nfact, meanlog=0, sdlog=.5), 
+                 -rlnorm(1, meanlog=0, sdlog=.5))
+        x@par[x@est] <- par[x@est]
+        x
+    }
+)
+
 setMethod(
     f = "GenRandomPars",
     signature = signature(x = 'GroupPars'),
@@ -554,6 +598,27 @@ P.comp <- function(par, Theta, ot = 0)
     return(.Call('partcompTraceLinePts', par, Theta))
 }
 
+setMethod(
+    f = "ProbTrace",
+    signature = signature(x = 'ideal', Theta = 'matrix'),
+    definition = function(x, Theta, useDesign = TRUE, ot=0){
+        return(P.ideal(x@par, Theta=Theta))
+    }
+)
+
+P.ideal <- function(par, Theta, ot = 0)
+{
+    alpha <- par[length(par)]
+    beta <- par[-length(par)]
+    eta <- -0.5*(Theta %*% beta + alpha)^2
+    eta <- ifelse(eta < -20, -20, eta)
+    P <- exp(eta)
+    P <- cbind((1-P), P)
+    P <- ifelse(P < 1e-20, 1e-20, P)
+    P <- ifelse(P > (1 - 1e-20), (1 - 1e-20), P)
+    return(P)
+}
+
 #----------------------------------------------------------------------------
 ## initialize for custom items
 setMethod("initialize",
