@@ -5,7 +5,7 @@
 #' prior distribution using equally spaced quadrature. EAP scores for models with more than
 #' three factors are generally not recommended since the integration grid becomes very large,
 #' resulting in slower estimation and less precision if the \code{quadpts} are too low.
-#' Therefore, MAP scores schould be used instead of EAP scores for higher dimensional models. 
+#' Therefore, MAP scores should be used instead of EAP scores for higher dimensional models. 
 #' Multiple imputation variants are possible for each estimator if a parameter 
 #' information matrix was computed, which are useful if the sample size/number of items were small.
 #'
@@ -53,6 +53,7 @@
 #' @param verbose logical; print verbose output messages?
 #' @param scores.only logical; return only the factor scores (only applicable when 
 #'   \code{full.scores = TRUE})
+#' @param ... additional arguments
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @keywords factor.scores
 #' @export fscores
@@ -62,7 +63,7 @@
 #'
 #' Thissen, D., Pommerich, M., Billeaud, K., & Williams, V. S. L. (1995). 
 #' Item Response Theory for Scores on Tests Including Polytomous Items with Ordered Responses. 
-#' \emph{Applied Psychological Measurement, 19}, 39-49
+#' \emph{Applied Psychological Measurement, 19}, 39-49.
 #'
 #' Warm, T. A. (1989). Weighted likelihood estimation of ability in item response theory.
 #' \emph{Psychometrika, 54}, 427-450.
@@ -104,7 +105,7 @@
 fscores <- function(object, rotate = '', full.scores = FALSE, method = "EAP",
                     quadpts = NULL, response.pattern = NULL,
                     returnER = FALSE, return.acov = FALSE, mean = NULL, cov = NULL, verbose = TRUE,
-                    scores.only = TRUE, full.scores.SE = FALSE, theta_lim = c(-6,6), MI = 0)
+                    scores.only = TRUE, full.scores.SE = FALSE, theta_lim = c(-6,6), MI = 0, ...)
 {
     if(is.null(quadpts))
         quadpts <- switch(as.character(object@nfact), '1'=61, '2'=31, '3'=15, '4'=9, '5'=7, 3)
@@ -112,6 +113,6 @@ fscores <- function(object, rotate = '', full.scores = FALSE, method = "EAP",
                             quadpts=quadpts, response.pattern=response.pattern,
                             verbose=verbose, returnER=returnER, gmean=mean, gcov=cov,
                             scores.only=scores.only, theta_lim=theta_lim, MI=MI,
-                            full.scores.SE=full.scores.SE, return.acov=return.acov)
+                            full.scores.SE=full.scores.SE, return.acov=return.acov, ...)
     ret
 }
