@@ -9,6 +9,7 @@
 #' @param x an estimated model x from the mirt package
 #' @param Theta a matrix containing the estimates of the latent trait scores 
 #'   (e.g., via \code{\link{fscores}})
+#' @param ... additional arguments to pass
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @keywords impute data
 #' @export imputeMissing
@@ -21,29 +22,39 @@
 #' for(i in 1:20)
 #'     dat[NAperson[i], NAitem[i]] <- NA
 #' (mod <- mirt(dat, 1))
-#' scores <- fscores(mod, method = 'MAP', full.scores = TRUE, scores.only = TRUE)
+#' scores <- fscores(mod, method = 'MAP', scores.only = TRUE)
 #'
 #' #re-estimate imputed dataset (good to do this multiple times and average over)
 #' fulldata <- imputeMissing(mod, scores)
 #' (fullmod <- mirt(fulldata, 1))
 #'
-#'
+#' #with multipleGroup
+#' group <- rep(c('group1', 'group2'), each=500)
+#' mod2 <- multipleGroup(dat, 1, group)
+#' fs <- fscores(mod2, full.scores=TRUE)
+#' fulldata2 <- imputeMissing(mod2, fs)
 #' }
-imputeMissing <- function(x, Theta){
+imputeMissing <- function(x, Theta, ...){
     if(is(x, 'MixedClass'))
         stop('mixedmirt xs not yet supported')
     if(is(x, 'MultipleGroupClass')){
         pars <- x@pars
-        group <- x@group
+        group <- x@Data$group
         data <- x@Data$data
-        for(i in 1L:length(pars)){
-            sel <- group == x@groupNames[i]
+        uniq_rows <- apply(data, 2L, function(x) list(sort(na.omit(unique(x)))))
+        for(g in 1L:length(pars)){
+            sel <- group == x@Data$groupNames[g]
             Thetatmp <- Theta[sel, , drop = FALSE]
-            data[sel, ] <- imputeMissing(pars[[i]], Thetatmp)
+            pars[[g]]@Data$data <- data[sel, ]
+            data[sel, ] <- imputeMissing(pars[[g]], Thetatmp, uniq_rows=uniq_rows)
         }
         return(data)
     }
+    dots <- list(...)
     pars <- x@pars
+    nfact <- pars[[1L]]@nfact
+    if(!is(Theta, 'matrix') || nrow(Theta) != nrow(x@Data$data) || ncol(Theta) != nfact)
+        stop('Theta must be a matrix of size N x nfact')
     K <- x@K
     J <- length(K)
     data <- x@Data$data
@@ -53,10 +64,10 @@ imputeMissing <- function(x, Theta){
         if(!any(is.na(data[,i]))) next
         P <- ProbTrace(x=pars[[i]], Theta=Theta)
         NAind <- Nind[is.na(data[,i])]
-        range <- c(min(data[,i], na.rm=TRUE), max(data[,i], na.rm=TRUE))
+        if(!is.null(dots$uniq_rows)) uniq <- dots$uniq_rows[[i]][[1L]]
+        else uniq <- sort(na.omit(unique(data[,i])))
         for(j in 1L:length(NAind))
-            data[NAind[j], i] <- sample(range[1L]:range[2L], 1, 
-                                        prob = P[NAind[j], , drop = FALSE])
+            data[NAind[j], i] <- sample(uniq, 1L, prob = P[NAind[j], , drop = FALSE])
     }
     return(data)
 }
